# -*- coding: UTF-8 -*-

# Specific Search: Add-on for searching specific text on virtual buffers
# Version: 1.0
# Date: 13/05/2012


import addonHandler
import globalPluginHandler
import api
import re
import os
import textInfos
import controlTypes
import gui
import wx
import ui
import speech
import cPickle
from cursorManager import CursorManager

addonHandler.initTranslation()

_basePath = os.path.dirname(os.path.abspath(__file__))
_savePath = os.path.join(_basePath, "search")


class GlobalPlugin(globalPluginHandler.GlobalPlugin):

	_lastSpecificFindText = ""
	_pickle = ""
	_states = []

	def standarFileName(self, fileName):
		notAllowed = re.compile("\?|:|\*|\t|<|>|\"|\/|\\||") # Invalid characters
		allowed = re.sub(notAllowed, "", unicode(fileName))
		return allowed

	def getFile(self):
		obj=api.getForegroundObject()
		file = obj.name
		file = file.rsplit(" - ", 1)[0]
		file = file.split("\\")[-1]
		file = self.standarFileName(file)
		file = file+".txt"
		path = os.path.join(_savePath, file)
		abspath=os.path.abspath(path)
		return abspath

	def getLastSpecificFindText(self):
		f = open(self.getFile(), "r")
		self._lastSpecificFindText = f.read()
		f.close()

	def saveSpecificFindTextDialog(self):
		try:
			self.getLastSpecificFindText()
		except IOError:
			self._lastSpecificFindText = ""
		d = wx.TextEntryDialog(gui.mainFrame, _("Type the text you wish to save, or delete text if you want to remove the previous saved search"), _("Save text for specific search"), defaultValue=self._lastSpecificFindText)
		def callback(result):
			if result == wx.ID_OK:
				# Make sure this happens after focus returns to the document.
				wx.CallLater(100, self.saveSpecificFindText, d.GetValue())
		gui.runScriptModalDialog(d, callback)

	def saveSpecificFindText(self, text):
		if not text:
			try:
				os.remove(self.getFile())
			except WindowsError:
				pass
			return
		f = open (self.getFile(), "w")
		f.write(text.encode("latin-1"))
		f.close

	def script_specificSave(self,gesture): 
		obj=api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if not (hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough):
			if not obj.role==52:
				gesture.send()
				return
		self.saveSpecificFindTextDialog()
	script_specificSave.__doc__ = _("Save a text string for a specific search")

	def doFindText(self,text,reverse=False):
		if not text:
			return
		obj=api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough:
			obj=treeInterceptor
		try:
			info=obj.makeTextInfo(textInfos.POSITION_CARET)
		except (NotImplementedError, RuntimeError):
			info=obj.makeTextInfo(textInfos.POSITION_FIRST)
		res=info.find(text,reverse=reverse)
		if res:
			obj.selection=info
			speech.cancelSpeech()
			info.move(textInfos.UNIT_LINE,1,endPoint="end")
			speech.speakTextInfo(info,reason=controlTypes.REASON_CARET)
		else:
			ui.message("not found")
		#CursorManager._lastFindText=text

	def doSpecificFindTextDialog(self):
		try:
			self.getLastSpecificFindText()
		except IOError:
			ui.message(_("File for specific search not found"))
			return
		self.doFindText(self._lastSpecificFindText)
		#gui.runScriptModalDialog(d, callback)

	def script_specificFind(self,gesture):
		obj=api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if not (hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough):
			if not obj.role==52:
				gesture.send()
				return
		self.doSpecificFindTextDialog()
	script_specificFind.__doc__ = _("find a text string from the current cursor position for a specific page")


	def getStates(self):
		self.getPickle()
		fileName = self._pickle
		try:
			self._states = cPickle.load(file(fileName, "r"))
		except IOError:
			self._states = []


	def getPickle(self):
		self._pickle = self.getFile().replace(".txt", ".pickle")

	def script_saveBookmark(self, gesture):
		self.getPickle()
		self.getStates()
		fileName = self._pickle
		obj = api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough:
			obj=treeInterceptor
		else:
			return
		start = obj.makeTextInfo(textInfos.POSITION_ALL)
		try:
			end = obj.makeTextInfo(textInfos.POSITION_CARET)
		except NotImplementedError:
			ui.message(_("Bookmark can not be saved"))
			return
		start.setEndPoint(end, "endToStart")
		count = len(start.text)
		state = count
		self._states.append(state)
		self._states = self._states[-1:]
		cPickle.dump(self._states, file(fileName, "wb"))
		ui.message(_("Saved %d" %count))


	def script_selectBookmark(self, gesture):
		self.getStates()
		try:
			bookmark = self._states[0]
		except IndexError:
			ui.message(_("Bookmark not found"))
			return
		obj = api.getFocusObject()
		treeInterceptor=obj.treeInterceptor
		if hasattr(treeInterceptor,'TextInfo') and not treeInterceptor.passThrough:
			obj=treeInterceptor
		else:
			return
		info = obj.makeTextInfo(textInfos.POSITION_FIRST)
		info.move(textInfos.UNIT_CHARACTER, bookmark)
		obj.selection = info
		info.move(textInfos.UNIT_LINE,1,endPoint="end")
		speech.speakTextInfo(info,reason=controlTypes.REASON_CARET)
		ui.message(_("Position: character %d") % bookmark)

	__gestures = {"kb:shift+.": "specificSave", "kb:.": "specificFind", "kb:control+NVDA+shift+k": "saveBookmark", "kb:control+alt+NVDA+k": "selectBookmark"}

